@@ -18,6 +18,10 @@
 #  include <stdlib.h>
 # endif
 #endif
+#include <sys/types.h>
+#ifdef HAVE_STDINT_H
+# include <stdint.h>
+#endif
 #ifdef HAVE_LOCALE_H
 # include <locale.h>
 #endif
@@ -48,7 +52,6 @@
 #  include <time.h>
 # endif
 #endif
-#include <sys/types.h>
 #include <sys/stat.h>
 #ifdef HAVE_FCNTL_H
 # include <fcntl.h>
@@ -255,7 +258,12 @@
 extern int errno;
 #endif
 #ifndef environ
+# ifdef __APPLE_CC__
+#  include <crt_externs.h>
+#  define environ (*_NSGetEnviron())
+# else
 extern char **environ;
+# endif
 #endif
 
 typedef struct AuthResult_ {
@@ -315,31 +323,8 @@ typedef enum {
 } ChannelProtectionLevel;
 
 int pureftpd_start(int argc, char *argv[], const char *home_directory);
-
-#ifdef __IPHONE__
-void pureftpd_register_login_callback(void (*callback)(void *user_data),
-                                      void *user_data);
-
-void pureftpd_register_logout_callback(void (*callback)(void *user_data),
-                                       void *user_data);
-
-void pureftpd_register_log_callback(void (*callback)(int crit,
-                                                     const char *message,
-                                                     void *user_data),
-                                    void *user_data);
-
-void pureftpd_register_simple_auth_callback(int (*callback)(const char *account,
-                                                            const char *password,
-                                                            void *user_data),
-                                            void *user_data);
-
-int pureftpd_shutdown(void);
-int pureftpd_enable(void);
-int pureftpd_disable(void);
-#endif
-int safe_write(const int fd, const void *buf_, size_t count);
 #ifdef WITH_TLS
-int secure_safe_write(void * const tls_fd, const void *buf_, size_t count);
+ssize_t secure_safe_write(void * const tls_fd, const void *buf_, size_t count);
 #endif
 void parser(void);
 void stripctrl(char * const buf, size_t len);
@@ -379,7 +364,7 @@ void dostou(void);
 void dofeat(void);
 void domlst(const char * const file);
 void dositetime(void);
-int ul_check_free_space(const char *name);
+int ul_check_free_space(const char *name, const double min_space);
 void mappedtov4(struct sockaddr_storage *ss);
 void disablesignals(void);
 void getnames(void);
@@ -509,9 +494,6 @@ the server may be insecure if a wrong value is set here.
 Your platform has a very large MAXPATHLEN, we should not trust it.
 #endif
 
-#ifdef __IPHONE__
-# define DEFAULT_MAX_USERS 1
-#endif
 #ifndef DEFAULT_MAX_USERS    
 # define DEFAULT_MAX_USERS 50
 #endif
@@ -777,7 +759,11 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 #  define setegid(X) (-1)
 # endif
 #endif
-    
+
+#if defined(HAVE_DEV_ARANDOM) || defined(HAVE_DEV_URANDOM) || defined(HAVE_DEV_RANDOM)
+# define HAVE_RANDOM_DEV 1
+#endif
+
 #define CRLF "\r\n"
 
 #ifdef WITH_DMALLOC
