@@ -8,6 +8,7 @@
 # include "log_mysql.h"
 # include "messages.h"
 # include "crypto.h"
+# include "alt_arc4random.h"
 
 # ifdef WITH_DMALLOC
 #  include <dmalloc.h>
@@ -322,7 +323,8 @@ void pw_mysql_check(AuthResult * const result,
     char *escaped_peer_ip = NULL;
     char *escaped_decimal_ip = NULL;    
     int committed = 1;
-    int crypto_crypt = 0, crypto_mysql = 0, crypto_md5 = 0, crypto_plain = 0;
+    int crypto_crypt = 0, crypto_mysql = 0, crypto_md5 = 0, crypto_sha1 = 0,
+        crypto_plain = 0;
     unsigned long decimal_ip_num = 0UL;
     char decimal_ip[42];
     char hbuf[NI_MAXHOST];
@@ -416,12 +418,15 @@ void pw_mysql_check(AuthResult * const result,
         crypto_crypt++;
         crypto_mysql++;
         crypto_md5++;
+        crypto_sha1++;
     } else if (strcasecmp(crypto, PASSWD_SQL_CRYPT) == 0) {
         crypto_crypt++;
     } else if (strcasecmp(crypto, PASSWD_SQL_MYSQL) == 0) {
         crypto_mysql++;
     } else if (strcasecmp(crypto, PASSWD_SQL_MD5) == 0) {
         crypto_md5++;
+    } else if (strcasecmp(crypto, PASSWD_SQL_SHA1) == 0) {
+        crypto_sha1++;
     } else {                           /* default to plaintext */
         crypto_plain++;
     }
@@ -470,7 +475,15 @@ void pw_mysql_check(AuthResult * const result,
             strcmp(crypted, spwd) == 0) {
             goto auth_ok;
         }
-    }    
+    }
+    if (crypto_sha1 != 0) {
+        const char *crypted;
+
+        if ((crypted = (const char *) crypto_hash_sha1(password, 1)) != NULL &&
+            strcmp(crypted, spwd) == 0) {
+            goto auth_ok;
+        }
+    }
     if (crypto_plain != 0) {
         if (*password != 0 &&    /* refuse null cleartext passwords */
             strcmp(password, spwd) == 0) {
